/* wcom.h */

#define PKT_READ_SIZE 5000

// NT driver custom ioctl numbers for special support calls.
#define CTL_CODE( DeviceType, Function, Method, Access ) (                 \
    ((DeviceType) << 16) | ((Access) << 14) | ((Function) << 2) | (Method) )
#define METHOD_BUFFERED                 0
#define FILE_ANY_ACCESS                 0
#define FILE_DEVICE_SERIAL_PORT         0x0000001b
#define IOCTL_RCKT_LOOPBACK_ON \
      CTL_CODE(FILE_DEVICE_SERIAL_PORT, 0x805,METHOD_BUFFERED,FILE_ANY_ACCESS)
#define IOCTL_RCKT_LOOPBACK_OFF \
      CTL_CODE(FILE_DEVICE_SERIAL_PORT, 0x806,METHOD_BUFFERED,FILE_ANY_ACCESS)

#define IOCTL_RCKT_SET_TOGGLE_LOW \
      CTL_CODE(FILE_DEVICE_SERIAL_PORT, 0x807,METHOD_BUFFERED,FILE_ANY_ACCESS)
#define IOCTL_RCKT_CLEAR_TOGGLE_LOW \
      CTL_CODE(FILE_DEVICE_SERIAL_PORT, 0x808,METHOD_BUFFERED,FILE_ANY_ACCESS)

// win95 escapecomm ioctl ids to set 485 toggle mode
#define RTSTOGGLE_ON 15654L
#define RTSTOGGLE_OFF 15655L


typedef struct {
  int  out_cnt;
  int  in_cnt;
  WORD error_flags;
  WORD event_flags;
} PortStatus;

class WCom {
public:

  HWND hwnd;        // handle of our port window
  WCom *next_wcom;  // keep linked list of all port windows
  HFONT hfont;
  char com_name[20];  // COM#
  HANDLE com_id;      // com port handle
  DWORD events;
  DWORD event_mask;   // bit flags of notifications we want
  int testing_flag;  // flag to stop main processing while testing

  int loop_supported;  // set if a port supports comtrol-loop id
  int num_buttons;     // number of buttons on window
  int ticks;
  unsigned int far *event_ptr;  // win16 event ptr from setcommeventmask
  int  ev_rec_cnt;   // win16,  rec-cnt notification amount
  int  ev_send_cnt;  // win16, send-cnt notification amount
  BYTE far *msr_cheat_ptr;  // win 16 "cheat" to get msr status.
  WORD msr_status;          // modem status reg(input pins)
  WORD mcr_status;          // modem control status(output pins)
  WORD old_msr_status;      // detect change in
  WORD old_mcr_status;
  unsigned char in_buf[PKT_READ_SIZE+2];
  int lights_update_flag;  // need to update the screen.

  int scr_size_x;
  int scr_size_y;

  DWORD ms_starttime;
  int cur_x;  // screen mouse position, current
  int cur_y;
 // extra Win32 baggage for overlapped(asynchronous mode)
 int         EventPendingFlag;
 OVERLAPPED  osEvent;     // for asynchronous operation

 OVERLAPPED  osWrite;     // for asynchronous operation
 int         OutPendingFlag;
 int         OutTotal;

 OVERLAPPED  osRead;      // for asynchronous operation
 int         InPendingFlag;
 int         InTotal;

  DCB dcb;               // our device control block, port-options

  COMMTIMEOUTS timeouts;  // win32 timeouts
 COMSTAT cur_com_stat;  // last reading.
 COMSTAT old_com_stat;  // recall so we can show changes

  WCom();
  ~WCom();
  void WCom::loop_test(void);
  void WCom::get_pnp_id(void);
  void WCom::check_com_port(void);
  int WCom::CheckEvents32(void);
  int WCom::OpenPort(char *name);
  int WCom::PortIdentify(void);
  int WCom::WritePort(unsigned char *str, int cnt);
  int WCom::ReadPort(unsigned char *str, int want_cnt, int *got_cnt);
  int WCom::StatusPort(PortStatus *ps);
  void WCom::update_lights(int all_flag);
  TTYwin ttywin;
  void WCom::print_comm_errors(unsigned int err);
  void WCom::print_modem_state(unsigned int modem_state);
  void WCom::print_events(unsigned int events);
  int WCom::LoopBack(int on);
};

extern char gtmparr[80];
extern HWND glob_hwnd;
extern HINSTANCE glob_hinst;      // current instance

extern WCom *head_wcom;

